unit PlugMain;

//=========================================================================
//
// I N T E R F A C E
//
//=========================================================================
interface

uses
  {genericas}
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, Menus, StdCtrls;

type

  //: Tipo para ejecutar un proc por RTTI con parametros estandard.
  TExecuteRoutine = procedure()of object;
  //: Tipo para obtener el idioma del plugin.
  TExecuteGetIdioma = function():string of object;

  //: Clase para el form principal 
  TFormMain = class(TForm)
    Button1: TButton;
    MainMenu1: TMainMenu;
    Hola1: TMenuItem;
    PlugIns1: TMenuItem;
    Cerrar1: TMenuItem;
    StatusBar1: TStatusBar;
    mmLog: TMemo;
    procedure Button1Click(Sender: TObject);
    procedure StatusBar1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    // path del dir. de plugins
    _pathP:String;
    // Lista de plugins encontrados
    _FilesList:TStringList;

    // mostrar mensaje en la ventana de log
    procedure _mens(Str:String);
    // Click en un elemento del menu de plugins.
    procedure _ClickElement (Sender: TObject);
    // Obtiene la cadena del idioma del plugin
    function _OntenerCadenaIdioma(AFileName:String):string;

  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

// constantes varias...
const
  PLUG_DIRNAME = 'AddIn';
  PLUG_MASK = '*.bpl';
  PLUG_EXTENSION = '.BPL';
  CHAR_BACKSLASH = '\';
  MASK_ALL_FILES = '*.*';
  CHAR_POINT = '.';
  PLUG_MAIN_FORM = 'TPagePlugIn';

// Cadena a traducir
const
  MSG_HOLA_MUNDO = 'Hola mundo!!';

//=========================================================================
//
// I M P L E M E N T A T I O N
//
//=========================================================================
implementation

{$R *.dfm}


uses
  {generales} ShellAPI;


//
procedure FindFiles(StartDir, FileMask: string; recursively: boolean; var FilesList: TStringList);
var
  SR: TSearchRec;
  DirList: TStringList;
  IsFound: Boolean;
  i: integer;
begin
  if (StartDir[length(StartDir)] <> CHAR_BACKSLASH) then begin
    StartDir := StartDir + CHAR_BACKSLASH;
  end;

  // Crear la lista de ficheos en el directorio StartDir (no directorios!)
  IsFound := FindFirst(StartDir + FileMask, faAnyFile - faDirectory, SR) = 0;
  // MIentras encuentre
  while IsFound do  begin
    FilesList.Add(StartDir + SR.Name);
    IsFound := FindNext(SR) = 0;
  end;

  FindClose(SR);

  // Recursivo?
  if (recursively) then begin
    // Build a list of subdirectories
    DirList := TStringList.Create;
    // proteccion
    try
      IsFound := FindFirst(StartDir + MASK_ALL_FILES, faAnyFile, SR) = 0;
      while IsFound do
      begin
        if ((SR.Attr and faDirectory) <> 0) and
          (SR.Name[1] <> CHAR_POINT) then
          DirList.Add(StartDir + SR.Name);
        IsFound := FindNext(SR) = 0;
      end;
      FindClose(SR);

      // Scan the list of subdirectories
      for i := 0 to DirList.Count - 1 do
        FindFiles(DirList[i], FileMask, recursively, FilesList);
    finally
      DirList.Free;
    end;
  end;
end;
//


procedure TFormMain.Button1Click(Sender: TObject);
begin
  // Estandard
  MessageDlg(MSG_HOLA_MUNDO, mtInformation, [mbOK], 0);
end;

procedure TFormMain.StatusBar1Click(Sender: TObject);
begin
  // Credits
  ShellExecute(Handle,
           'open',
           'http://neftali.clubdelphi.com',
           nil,
           nil,
           SW_SHOW);
end;

procedure TFormMain.FormShow(Sender: TObject);
var
  Str, cName:String;
  i:Integer;
begin

  // Ocultar el men
  PlugIns1.Visible := False;
  _FilesList := TStringList.Create();

  // Detectando plugins
  _mens('Directorio de plug-ins...');
  _pathP := IncludeTrailingBackSlash(ExtractFilePath(Application.ExeName));
  _pathP := IncludeTrailingBackSlash(_pathP + PLUG_DIRNAME);

  // Crearlo si no existe
  ForceDirectories(_pathP);
  _mens('Creando directorio si es necesario...');

  // Buscar plugIns
  _mens('Buscando plug-ins instalados...');  
  FindFiles(_pathP, PLUG_MASK, False, _FilesList);

  // No encontrados
  if (_FilesList.Count = 0) then begin
    _mens('No se han encontrado plug-ins.');
    Exit;
  end
  else begin
    PLugIns1.Visible := true;
    _mens(Format('Encontrados %d plug-ins.',[_FilesList.Count]));
  end;

  // Cargarlos...
  for i := 0 to (_FilesList.Count - 1) do begin
    // Aadir los nombres...
    Str := ExtractFileName(_FilesList.Strings[i]);
    // obtener el idioma
    cName := _OntenerCadenaIdioma(_FilesList.Strings[i]);
    // Nombre
    Str := ChangeFileExt(Str, '');
    // aadirlo al menu
    PLugIns1.Add(NewItem(cName, 0, False, True, _ClickElement, 0, Str));
    _mens('Encontrado plug-in: ' + _FilesList.Strings[i]);
  end;


end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin

  // Asignado?
  if Assigned(_FilesList) then begin
    FreeAndNil(_FilesList);
  end;

end;
//: mostrar mensaje en la ventana de log.
procedure TFormMain._mens(Str: String);
begin
  mmLog.Lines.Add(Str);
end;

// Se lanza cuando seleccionados un plugin del menu.
procedure TFormMain._ClickElement(Sender: TObject);
var
  Str:String;
  hndl:HMODULE;
  AClass:TPersistentClass;
  AForm:TForm;
  Routine: TMethod;
begin

  // Cual es el elemento pulsado?
  Str := TMenuItem(Sender).Name;
  // Buscar el fichero
  Str := Str + PLUG_EXTENSION;
  Str := _pathP + Str;
  // Existe?
  if FileExists(Str) then begin
    // Cargar
    hndl := LoadPackage(Str);

    // Accder a la clase del menu
    AClass := GetClass(PLUG_MAIN_FORM);

    // Bloque de proteccion para la carga
    try
      // No ha podido acceder a ella
      if (AClass <> nil) then begin
        AForm := TComponentClass(AClass).Create(Application) as TForm;

        // proteccion para liberar
        try
          // Acceso a la rutina
          Routine.Data := Pointer(AForm);
          // Returns the address of a published method.
          Routine.Code := (AForm).MethodAddress('Traduce');

          // Encontrada la rutina de acceso
          if (Routine.Code = nil) then begin
            MessageDlg('Error al cargar el plug-in', mtError, [mbOK], 0);MessageDlg('Error, no se ha encontrado el procedimiento (Execute) para el correcto funcionamieno del Plug-in.', mtError, [mbOK], 0);
            Exit;
          end;

          // ejecutar
          TExecuteRoutine(Routine)();

        finally
          Aform.Free;
        end;

        // Liberar / Descargar
        UnloadPackage(hndl);
      end
      else begin
        MessageDlg('La clase para acceder al plug-in parece que no est correctamente registrada.', mtError, [mbOK], 0);
      end;
    except
      on E:Exception do begin
        MessageDlg('Error al cargar el plug-in.', mtError, [mbOK], 0);
      end;
    end;
  end;

end;

// Obtiene la cadena del idioma del plugin.
function TFormMain._OntenerCadenaIdioma(AFileName:String): string;
var
  Str:String;
  hndl:HMODULE;
  AClass:TPersistentClass;
  AForm:TForm;
  Routine: TMethod;
begin

  // Cargar
  hndl := LoadPackage(AFileName);

  // Accder a la clase del menu
  AClass := GetClass(PLUG_MAIN_FORM);

  // Bloque de proteccion para la carga
  try
    // No ha podido acceder a ella
    if (AClass <> nil) then begin
      AForm := TComponentClass(AClass).Create(Application) as TForm;

      // proteccion para liberar
      try
        // Acceso a la rutina
        Routine.Data := Pointer(AForm);
        // Returns the address of a published method.
        Routine.Code := (AForm).MethodAddress('Idioma');

        // Encontrada la rutina de acceso
        if (Routine.Code = nil) then begin
          MessageDlg('Error al cargar el plug-in', mtError, [mbOK], 0);MessageDlg('Error, no se ha encontrado el procedimiento (Execute) para el correcto funcionamieno del Plug-in.', mtError, [mbOK], 0);
          Exit;
        end;

        // ejecutar
        Result := TExecuteGetIdioma(Routine)();

      finally
        Aform.Free;
      end;

      // Liberar / Descargar
      UnloadPackage(hndl);
    end
    else begin
      MessageDlg('La clase para acceder al plug-in parece que no est correctamente registrada.', mtError, [mbOK], 0);
    end;
  except
    on E:Exception do begin
      MessageDlg('Error al cargar el plug-in.', mtError, [mbOK], 0);
    end;
  end;

end;

end.

